#!/usr/local/perl/bin/perl
#------------------------------------------------------------------------------
#This software is in the public domain, furnished "as is", without technical
#support, and with no warranty, express or implied, as to its usefulness for
#any purpose.
#
# http.pl
# This perl script creates a data page for the FX Data Monitor.  It is 
# run from a cron job under the FXA account.
#
#Author: Ron Kahn (modified by Jordan Gerth for local use on 7/29/2007)
#--------------------------------------------------------------------------

unshift ( @INC, "/home/fxa/bin" );
use NetCDF;
use HTTP::Date;
require "getopts.pl";
require "logNCF.pl";

#--------------------------------------------------------------------------
# get a shortened name for the program for use with error messages
#--------------------------------------------------------------------------
@programName = split('/',$0);
$0 = $programName[$#programName];
$shortName = (split(/\./,$0))[0];
 
$dataType = $ARGV[$#ARGV]; # Which data is the process running on?  After all it runs on grids, graphics, satellite, etc.
$dataType =~ s/\s//g;      # eliminate the white space.

#--------------------------------------------------------------------------
# Name the logfile... 
#--------------------------------------------------------------------------
$logfile = "$shortName"."\.log";

#--------------------------------------------------------------------------
# redirect STDERR and STDOUT
#--------------------------------------------------------------------------
open (STDOUT, ">>$ENV{\"LOG_DIR\"}/$logfile" ) || die ($date=(split(/\n/,`date`))[0]," $0:$dataType:ERROR...Can't open STDOUT.\n");
open (STDERR, ">&STDOUT") || die ($date=(split(/\n/,`date`))[0]," $0:$dataType:ERROR...Can't open STDERR.\n");

#--------------------------------------------------------------------------
# Create a new log file if necessary and redirect STDERR and STDOUT to point
# to that file. Only do it when grids are being processed
# otherwise at 00Z several new logfiles will be created, and the older ones
# will disappear! 
#--------------------------------------------------------------------------
$currtime = time();
($min,$hour) = (gmtime($currtime))[1,2];
if ($dataType =~ /GridData/) {
   if(($hour == 0) && ($min <= 9)){
      if (-e "$ENV{\"LOG_DIR\"}/$logfile") {
         close(STDOUT);
         close(STDERR);
         }
      rename("$ENV{\"LOG_DIR\"}/$logfile\.1","$ENV{\"LOG_DIR\"}/$logfile\.2");
      rename("$ENV{\"LOG_DIR\"}/$logfile","$ENV{\"LOG_DIR\"}/$logfile\.1");
      open (STDOUT, ">>$ENV{\"LOG_DIR\"}/$logfile" ) || 
                                           die ($date=(split(/\n/,`date`))[0]," $0:$dataType:ERROR...Can't open STDOUT.\n");
      open (STDERR, ">&STDOUT") || die  ($date=(split(/\n/,`date`))[0]," $0:$dataType:ERROR...Can't open STDERR.\n");
      print STDOUT ("$logfile opened on ", $date=(split(/\n/,`date`))[0], "\n");
      }
   }
#--------------------------------------------------------------------------
# Make sure all the buffers for both STDOUT and STDERR are flushed.
#--------------------------------------------------------------------------
select (STDOUT);
$| = 1;
select (STDERR);
$| = 1;

#--------------------------------------------------------------------------
# Initialize the necessary variables
#--------------------------------------------------------------------------
$ENV{"TZ"} = "GMT";
$usage = "$0 -c FileList -o OutputHTMLFile -h PageHeader [-v]\n";
%Summary = ();
%Exp = ();
%Attnage = ();
%Errage = ();
%Mtime = ();
%DocURL = ();
%Percentage = ();
%InformNCF = ();
$timenow = time;

#--------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------
do Getopts('c:o:h:d:v');

#--------------------------------------------------------------------------
# -c specified the files to check
#--------------------------------------------------------------------------
unless ($opt_c) 
    {
    print STDOUT "\nMust supply -c argument\n"; 
    print STDOUT $usage; 
    exit 1;
    }

#--------------------------------------------------------------------------
# is the file readable?
#--------------------------------------------------------------------------
unless (-r $opt_c)
    {
    print STDOUT "\nFile List is not readable\n"; 
    print STDOUT $usage;
    exit 1;
    }

#--------------------------------------------------------------------------
# -o specified the HTML file to output
#--------------------------------------------------------------------------
unless ($opt_o) 
    {
    print STDOUT "\nMust supply -o argument\n"; 
    print STDOUT $usage; 
    exit 1;
    }

chop($host = `hostname`);
$retcode = $? >> 8;
if (!($retcode == 0)) {
    warn ($date=(split(/\n/,`date`))[0], " PROBLEM...Unable to resolve hostname.\n");
    }
#$host2 = (split(/\./, $host))[0];
#$host2 = (split(/-/, $host))[0];
@a = split "/", $opt_c;
$data_type = pop @a;
$data_type = (split(/\./, $data_type))[0];

#--------------------------------------------------------------------------
# Read in what the state of each data set was from the previously data 
# monitor run.
#--------------------------------------------------------------------------
open (STATE_FIL,"$ENV{\"FXA_HOME\"}/data/$data_type\.state") || 
                                         warn ($date=(split(/\n/,`date`))[0]," $0:$dataType:ERROR...Can't open $data_type.state \n");
while ($oldState = <STATE_FIL>) {
     @stateInfo = split("::",$oldState);
     $states{$stateInfo[0]} = $stateInfo[1];
     }
close (STATE_FIL);
chop(%states);

#--------------------------------------------------------------------------
# open the file to read file expressions and other config information
#--------------------------------------------------------------------------
open(FIL, "$opt_c");
$count = 0;
for ($indx=0; <FIL>; $indx++) {
    next if /^!/ || /^\s/;

#--------------------------------------------------------------------------
# remove trailing <CR><LF>
#--------------------------------------------------------------------------
    chop;

    s/#(.*)/\1/;                       # drop the leading pound sign...
    s/\t//g;                           # remove tabs if they exist

    #
#--------------------------------------------------------------------------
# split out record into file expression, long name, 
# age for attn, age for error, Info Page name, and whether to notify the NCF.
#--------------------------------------------------------------------------

    (@name[$count], $exp, $attnage, $errage, $docurl, $informNCF) = split('#', $_);
    $Summary{@name[$count]} = -1;      # init the entry to "unknown"
    $exp =~ s/\s//g;                   # allow no whitespace in filename
    $exp = $ENV{"FXA_DATA"} . $exp;    # concatonate with the data directory 
    $Exp{@name[$count]} = $exp;
    $Attnage{@name[$count]} = $attnage;
    $Errage{@name[$count]} = $errage;
    $Mtime{@name[$count]} = -1;        
    $DocURL{@name[$count]} = $docurl;
    $InformNCF{@name[$count]} = $informNCF;
    $docurl = "";
    $Percentage{@name[$count]} = -1;        
    $count++;
    }
close(FIL);

#--------------------------------------------------------------------------
# loop through all matching files in the directory finding the newest
#--------------------------------------------------------------------------
foreach $name (@name)
    {
#--------------------------------------------------------------------------
#  Find the latest modified file.
#--------------------------------------------------------------------------
    @file_names = `ls -lrt $Exp{$name} 2> /dev/null`;
    $retcode = $? >> 8;
    if (!($retcode == 0)) {
       warn ($date=(split(/\n/,`date`))[0], " $0:$dataType:PROBLEM...Error returned \"$!\" from ls call to $name.\n");
       }

    $count = $#file_names;
    while ($file_names[$count]=~/^d/){$count--;}
    $filename = $file_names[$count];
    $filename =~ s/.* //;
    if ($filename ne "") {
       chop($filename);
       if ($filename !~ $Exp{$name}){$filename = $Exp{$name} . $filename;}
       $mtime = (stat($filename))[9];
       $fileage = $timenow - $mtime;
       $Summary{$name} = $fileage;
       $Mtime{$name} = $mtime;
       }
#--------------------------------------------------------------------------
#  Summary{$name} should have the latest file in it, provided it found one.
#--------------------------------------------------------------------------
     if ($data_type =~ /grid/) {
        if ($Summary{$name} >= 1) {
           $percent = &getInvPercent($filename);
           $Percentage{$name} = $percent;
           }
        }
     if ($data_type =~ /local/) {
        if ((($name =~ /LAPS/) || ($name =~ /MAPS/))){
           if ($Summary{$name} >= 1) {
              $percent = &getInvPercent($filename);
              $Percentage{$name} = $percent;
              }
           }
        }
    }

#--------------------------------------------------------------------------
# Calculate Expiration date for the html page.
#--------------------------------------------------------------------------
$expireTime = time2str(time+600);

$date = `date`;
chop($date);

#--------------------------------------------------------------------------
# start printing html
#--------------------------------------------------------------------------
open( OUT, ">$opt_o" ) || die ($date=(split(/\n/,`date`))[0]," $0:$dataType:ERROR...can't open html output file\n");

print OUT "<table border=1 cellpadding=1 cellspacing=0 width=600>\n";
print OUT "<caption><h1>$opt_h on $host</h1>\n";
print OUT "<h4>Last modified $date</h4></caption>\n";
print OUT "<tr>\n";
print OUT "<th width=75>State</th>\n";
print OUT "<th width=250>Last Update</th>\n";
print OUT "<th width=275>Description</th>\n";
print OUT "</tr>\n\n";

$state_val = 0;
#--------------------------------------------------------------------------
# Save the state of the data.
#--------------------------------------------------------------------------

open (STATE_FIL,">$ENV{\"FXA_HOME\"}/data/$data_type\.state") || 
                                    warn ($date=(split(/\n/,`date`))[0], " $0:$dataType:PROBLEM...Can't write to $data_type.state \n");


#--------------------------------------------------------------------------
# print results
#--------------------------------------------------------------------------
foreach $name (sort keys(%Summary))
    {
    if ($Mtime{$name} == -1)
        {
        $state = '<IMG SRC="img/unknown.gif" ALT="[UNKNOWN]">';
        print STATE_FIL "$name\::UNKNOWN\n";
        }
    elsif ($Summary{$name} < $Attnage{$name}) 
        {
        if ($state_val < 1) {$state_val = 1;}
        $state = '<IMG SRC="img/yes.gif" ALT="[UP]">';
        print STATE_FIL "$name\::UP\n";
        if (index($states{$name},"DOWN") >= 0){
            if ($InformNCF{$name} > 0 ) {
              logDataState($name,"Green");
              warn ($date=(split(/\n/,`date`))[0], "WARNING...State change occurred from DOWN to UP.\n");
               }
            }
        if (($Percentage{$name} > 0) && ($Percentage{$name} <= 7.5)) {
           $state = '<IMG SRC="img/yes05.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 7.5) && ($Percentage{$name} <= 12.5)) {
           $state = '<IMG SRC="img/yes10.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 12.5) && ($Percentage{$name} <= 17.5)) {
           $state = '<IMG SRC="img/yes15.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 17.5) && ($Percentage{$name} <= 22.5)) {
           $state = '<IMG SRC="img/yes20.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 22.5) && ($Percentage{$name} <= 27.5)) {
           $state = '<IMG SRC="img/yes25.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 27.5) && ($Percentage{$name} <= 32.5)) {
           $state = '<IMG SRC="img/yes30.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 32.5) && ($Percentage{$name} <= 37.5)) {
           $state = '<IMG SRC="img/yes35.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 37.5) && ($Percentage{$name} <= 42.5)) {
           $state = '<IMG SRC="img/yes40.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 42.5) && ($Percentage{$name} <= 47.5)) {
           $state = '<IMG SRC="img/yes45.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 47.5) && ($Percentage{$name} <= 52.5)) {
           $state = '<IMG SRC="img/yes50.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 52.5) && ($Percentage{$name} <= 57.5)) {
           $state = '<IMG SRC="img/yes55.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 57.5) && ($Percentage{$name} <= 62.5)) {
           $state = '<IMG SRC="img/yes60.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 62.5) && ($Percentage{$name} <= 77.5)) {
           $state = '<IMG SRC="img/yes65.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 67.5) && ($Percentage{$name} <= 72.5)) {
           $state = '<IMG SRC="img/yes70.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 72.5) && ($Percentage{$name} <= 77.5)) {
           $state = '<IMG SRC="img/yes75.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 77.5) && ($Percentage{$name} <= 82.5)) {
           $state = '<IMG SRC="img/yes80.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 82.5) && ($Percentage{$name} <= 87.5)) {
           $state = '<IMG SRC="img/yes85.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 87.5) && ($Percentage{$name} <= 92.5)) {
           $state = '<IMG SRC="img/yes90.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 92.5) && ($Percentage{$name} <= 99.9)) {
           $state = '<IMG SRC="img/yes95.gif" ALT="[UP]">';
           }
        elsif (($Percentage{$name} > 99.9) && ($Percentage{$name} <= 100)) {
           $state = '<IMG SRC="img/yes100.gif" ALT="[UP]">';
           }
        }
    elsif (($Summary{$name} >= $Attnage{$name}) && 
           ($Summary{$name} <= $Errage{$name})) 
        {
        $state = '<IMG SRC="img/caution.gif" ALT="[ATTN]">';
        print STATE_FIL "$name\::ATTN\n";
        if (index($states{$name},"DOWN") >= 0){
            if ($InformNCF{$name} > 0) {
              logDataState($name,"Yellow");
              warn ($date=(split(/\n/,`date`))[0], "WARNING...State change occurred from DOWN to ATTN.\n");
               }
            }
        if ($state_val < 2) {$state_val = 2;}
        }
    elsif ($Summary{$name} > $Errage{$name}) 
        {
        $state = '<IMG SRC="img/no.gif" ALT="[DOWN]">';
        print STATE_FIL "$name\::DOWN\n";
        if ((index($states{$name},"UP") >= 0) || (index($states{$name},"ATTN") >= 0)) {
            if ($InformNCF{$name} > 0) {
              logDataState($name,"Red");
              warn ($date=(split(/\n/,`date`))[0], "WARNING...State change occurred from UP or ATTN to DOWN.\n");
               }
            }
        $state_val = 3;
        }
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) 
      = gmtime($Mtime{$name});
    print OUT "<tr>\n";
    print OUT "<th>$state</th>\n";
    if ($Mtime{$name} != -1)
       {
        printf OUT "<th> %02d\/%02d\/%02d %02d:%02d:%02d </th>\n",
             $year+1900, $mon + 1, $mday, $hour, $min, $sec;
       }
    else
       {
        print OUT "<th>---</th>\n";
       }
    print OUT "<td align=\"left\"><strong>$name</strong></td>\n";
    print OUT "</tr>\n";
    }

print OUT "</table>\n";
close(OUT);
close(STATE_FIL);

#--------------------------------------------------------------------------
# Read in Configuration Information.
#--------------------------------------------------------------------------
open (FIL, "$ENV{\"FXA_HOME\"}/bin-local/dataMon.cfg") ||
                                       warn ($date=(split(/\n/,`date`)
)[0]," $0:ERROR...Can't open dataMon.cfg\n");
chop(@configInfo = <FIL>);
for ($cnt=0;$cnt<@configInfo;$cnt++){
    $configInfo[$cnt] =~ s/^!.*//g;
    }
@SERVER_DIRECTORY = grep(/SERVER_DIRECTORY/,@configInfo);
$SERVER_DIRECTORY = $SERVER_DIRECTORY[0];
$SERVER_DIRECTORY =~ s/ |\t//g;
@SERVER_DIRECTORY = split(/:| /,$SERVER_DIRECTORY);

#--------------------------------------------------------------------------
# Copy html files to proper directories. 
#--------------------------------------------------------------------------
$dir = $SERVER_DIRECTORY[1] ."dataMon/html/";
`cp -p $opt_o $dir`;
$retcode = $? >> 8;
if (!($retcode == 0)) { warn ($date = (split(/\n/,`date`))[0], " $0:$dataType:PROBLEM...Error returned \"$!\" from cp call. \n");}

sub getInvPercent {
   $ncid = 0;
   $ncid = NetCDF::open("@_",RD);
   NetCDF::inquire($ncid,$ndims,$nvars,$natts,$recdim);
   NetCDF::diminq($ncid,$recdim,$dimname,$nrec);
   $valtime_id = NetCDF::dimid($ncid,"n_valtimes");
   NetCDF::diminq($ncid,$valtime_id,$dimname,$nval_times);
   $numGridsPossible = 0;
   $numGridsAvailable = 0;
   $gridsPerRecord = 0;
   for ($recs=0;$recs<$nrec;$recs++) {
      for ($varcount=0;$varcount<$nvars;$varcount++) {
         NetCDF::varinq($ncid,$varcount,$varname,$type,$dims,\@dimids,$atts);
         $position = index($varname,"Inventory");
         if ($position >= 0) {
            NetCDF::diminq($ncid,$dimids[1],$dimname,$dimsize);
            if ($recs == 0) {
               $gridsPerRecord += $dimsize;
               }
            @start = ($recs,0);
            @count = (1,$dimsize);;
            NetCDF::varget($ncid,$varcount,\@start,\@count,\@values);
            for ($val=0;$val<$dimsize;$val++) {
               if ($values[$val] != 0) {
                  $numGridsAvailable += 1;
                  }
               }
            }
         }
      }
   NetCDF::close($ncid);
   $numGridsPossible = $gridsPerRecord * $nval_times; 
   if ($numGridsPossible == 0){$numGridsPossible = 1;}
   $numGridsAvailable / $numGridsPossible * 100;
   }
